@@ -8,9 +8,9 @@
 
 #include "osmaddress.h"
 #include "osmelement.h"
+#include "platformutil.h"
 
 #include <KOSMIndoorMap/MapData>
-#include <KOSMIndoorMap/PlatformModel>
 
 #include <QQmlEngine>
 
@@ -24,10 +24,10 @@ void KOSMIndoorMapQuickPlugin::registerTypes(const char *uri)
     qRegisterMetaType<MapData>();
     qRegisterMetaType<OSMAddress>();
     qRegisterMetaType<OSMElement>();
-    qRegisterMetaType<Platform>();
-    qRegisterMetaType<Platform::Mode>();
 
-    qmlRegisterUncreatableMetaObject(Platform::staticMetaObject, "org.kde.kosmindoormap", 1, 0, "Platform", {});
+    qmlRegisterSingletonType("org.kde.kosmindoormap", 1, 0, "PlatformUtil", [](QQmlEngine*, QJSEngine *engine) -> QJSValue {
+        return engine->toScriptValue(KOSMIndoorMap::PlatformUtil());
+    });
 }
 
 #include "moc_kosmindoormapquickplugin.cpp"
